@@ -514,7 +514,7 @@ const FieldTypes* ConnectionPool::get_field_types() const
   return m_pFieldTypes;
 }
 
-Gnome::Gda::SqlOperatorType ConnectionPool::get_string_find_operator() const
+Glib::ustring ConnectionPool::get_string_find_operator() const
 {
   g_assert(m_backend.get());
   return m_backend->get_string_find_operator();
