@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "privileges.h"
+#include <libglom/data_structure/privileges.h>
 
 namespace Glom
 {
