@@ -25,8 +25,10 @@
 #include <libglom/privs.h>
 #include <libglom/data_structure/parameternamegenerator.h>
 #include <libglom/utils.h>
+#include <libgdamm/value.h>
 #include <libgda/libgda.h> // For gda_g_type_from_string
 #include <glibmm/i18n.h>
+#include <iostream>
 
 namespace Glom
 {
@@ -479,7 +481,7 @@ void set_database_preferences(Document* document, const SystemPrefs& prefs)
     builder->add_field_value(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_LOGO, prefs.m_org_logo);
   }
   builder->set_where(builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-                                       builder->add_id(GLOM_STANDARD_TABLE_PREFS_FIELD_ID),
+                                       builder->add_field_id(GLOM_STANDARD_TABLE_PREFS_FIELD_ID, GLOM_STANDARD_TABLE_PREFS_TABLE_NAME),
                                        builder->add_expr(1)));
   const bool test = query_execute(builder);
 
@@ -523,7 +525,7 @@ bool add_standard_tables(Document* document)
           builder->set_table(GLOM_STANDARD_TABLE_PREFS_TABLE_NAME);
           builder->add_field_value(GLOM_STANDARD_TABLE_PREFS_FIELD_NAME, system_name);
           builder->set_where(builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-                                               builder->add_id(GLOM_STANDARD_TABLE_PREFS_FIELD_ID),
+                                               builder->add_field_id(GLOM_STANDARD_TABLE_PREFS_FIELD_ID, GLOM_STANDARD_TABLE_PREFS_TABLE_NAME),
                                                builder->add_expr(1)));
           const bool test = query_execute(builder);
           if(!test)
@@ -675,6 +677,98 @@ bool add_standard_groups(Document* document)
   return true;
 }
 
+bool add_groups_from_document(Document* document)
+{
+  Glib::RefPtr<Gnome::Gda::Connection> gda_connection = get_connection();
+  if(!gda_connection)
+  {
+    std::cerr << "add_standard_groups(): No connection yet." << std::endl;
+  }
+
+  //Get the list of groups from the database server:
+  const type_vec_strings database_groups = Privs::get_database_groups();
+
+  //Get the list of groups from the document:
+  const Document::type_list_groups document_groups = document->get_groups();
+
+  //Add each group if it doesn't exist yet:
+  for(Document::type_list_groups::const_iterator iter = document_groups.begin();
+    iter != document_groups.end(); ++iter)
+  {
+    const GroupInfo& group = *iter;
+    const Glib::ustring name = group.get_name();
+
+    //See if the group exists in the database:
+    type_vec_strings::const_iterator iterFind = std::find(database_groups.begin(), database_groups.end(), name);
+    if(!name.empty() && iterFind == database_groups.end())
+    {
+      Glib::ustring query = "CREATE GROUP \"" + name  + "\"";
+
+      //The "SUPERUSER" here has no effect because SUPERUSER is not "inherited" to member users.
+      //But let's keep it to make the purpose of this group obvious.
+      if(group.m_developer)
+        query += " WITH SUPERUSER";
+
+      const bool test = query_execute_string(query);
+      if(!test)
+      {
+        std::cerr << G_STRFUNC << ": CREATE GROUP failed when adding the group with name=" << name << std::endl;
+        return false;
+      }
+    }
+  }
+
+  return true;
+}
+
+bool set_table_privileges_groups_from_document(Document* document)
+{
+  Glib::RefPtr<Gnome::Gda::Connection> gda_connection = get_connection();
+  if(!gda_connection)
+  {
+    std::cerr << "add_standard_groups(): No connection yet." << std::endl;
+  }
+
+  //Get the list of groups from the database server:
+  const type_vec_strings database_groups = Privs::get_database_groups();
+
+  //Get the list of groups from the document:
+  const Document::type_list_groups document_groups = document->get_groups();
+
+  //Get the list of tables:
+  const Document::type_listTableInfo table_list = document->get_tables();
+
+  bool result = true;
+
+  for(Document::type_list_groups::const_iterator iter = document_groups.begin();
+    iter != document_groups.end(); ++iter)
+  {
+    const GroupInfo& group_info = *iter;
+    const Glib::ustring group_name = group_info.get_name();
+
+    //See if the group exists in the database:
+    type_vec_strings::const_iterator iterFind = std::find(database_groups.begin(), database_groups.end(), group_name);
+    if(!group_name.empty() && iterFind == database_groups.end())
+    {
+      std::cerr << G_STRFUNC << ": group does not exist in the database. group name=" << group_name << std::endl;
+      result = false;
+      continue;
+    }
+
+    //Look at each table privilege for this group:
+    for(GroupInfo::type_map_table_privileges::const_iterator iter = group_info.m_map_privileges.begin();
+      iter != group_info.m_map_privileges.end(); ++iter)
+    {
+      const Glib::ustring table_name = iter->first;
+      const Privileges& privs = iter->second;
+
+      //Set the table privilege for the group:
+      Privs::set_table_privileges(group_name, table_name, privs, group_info.m_developer);
+    }
+  }
+
+  return result;
+}
 
 namespace { //anonymous
 
@@ -798,7 +892,7 @@ type_vec_fields get_fields_for_table_from_database(const Glib::ustring& table_na
     holder_table_name->set_value(quoted_table_name, error);
 #endif
 
-    std::list< Glib::RefPtr<Gnome::Gda::Holder> > holder_list;
+    std::vector< Glib::RefPtr<Gnome::Gda::Holder> > holder_list;
     holder_list.push_back(holder_table_name);
 
     Glib::RefPtr<Gnome::Gda::DataModel> data_model_fields;
@@ -1295,10 +1389,10 @@ Gnome::Gda::Value get_next_auto_increment_value(const Glib::ustring& table_name,
   builder->add_field_value_as_value(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE, next_value);
   builder->set_where(builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_AND,
                                        builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-                                                         builder->add_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME),
+                                                         builder->add_field_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME),
                                                          builder->add_expr(table_name)),
                                        builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-                                                         builder->add_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME),
+                                                         builder->add_field_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME),
                                                          builder->add_expr(field_name))));
   const bool test = query_execute(builder);
   if(!test)
@@ -1326,10 +1420,10 @@ Gnome::Gda::Value auto_increment_insert_first_if_necessary(const Glib::ustring&
   builder->set_where(
     builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_AND,
       builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-        builder->add_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME),
+        builder->add_field_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME),
         builder->add_expr(table_name)),
       builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-        builder->add_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME),
+        builder->add_field_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME),
         builder->add_expr(field_name))));
 
   const Glib::RefPtr<const Gnome::Gda::DataModel> datamodel = query_execute_select(builder);
@@ -1383,9 +1477,9 @@ static void recalculate_next_auto_increment_value(const Glib::ustring& table_nam
 
   //Get the max key value in the database:
   Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
-  std::list<guint> args;
-  args.push_back(builder->add_id(table_name + "." + field_name));
-  builder->add_field_id(builder->add_function("MAX", args));
+  std::vector<guint> args;
+  args.push_back(builder->add_field_id(field_name, table_name));
+  builder->add_field_value_id(builder->add_function("MAX", args));
   builder->select_add_target(table_name);
 
   Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute_select(builder);
@@ -1410,10 +1504,10 @@ static void recalculate_next_auto_increment_value(const Glib::ustring& table_nam
     builder->add_field_value_as_value(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE, next_value);
     builder->set_where(builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_AND,
                                          builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-                                                           builder->add_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME),
+                                                           builder->add_field_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME),
                                                            builder->add_expr(table_name)),
                                          builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-                                                           builder->add_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME),
+                                                           builder->add_field_id(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME),
                                                            builder->add_expr(field_name))));
 
     const bool test = query_execute(builder);
