@@ -315,6 +315,7 @@ private:
   void on_maemo_touchselector_changed(int column);
   #else
   void treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index, int data_model_column_index);
+  void on_treeview_cell_editing_started(Gtk::CellEditable* cell_editable, const Glib::ustring& path);
   #endif //GLOM_ENABLE_MAEMO
 
 
@@ -322,6 +323,7 @@ private:
   #ifndef GLOM_ENABLE_MAEMO
   virtual void on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index, int data_model_column_index);
   virtual void on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index, int data_model_column_index);
+  void on_idle_treeview_cell_edited_revert(const Gtk::TreeModel::Row& row, guint model_column_index);
 
   bool on_treeview_column_drop(Gtk::TreeView* treeview, Gtk::TreeViewColumn* column, Gtk::TreeViewColumn* prev_column, Gtk::TreeViewColumn* next_column);
   void on_treeview_columns_changed();
@@ -345,7 +347,8 @@ private:
   virtual bool on_treeview_columnheader_button_press_event(GdkEventButton* event);
   virtual void on_treeview_column_clicked(int model_column_index);
   void on_treeview_column_resized(int model_column_index, DbTreeViewColumnGlom* view_column);
-  virtual void on_cell_button_clicked(const Gtk::TreeModel::Path& path);
+  void on_idle_row_edit();
+  void on_cell_button_clicked(const Gtk::TreeModel::Path& path);
   void on_cell_layout_button_clicked(const Gtk::TreeModel::Path& path, int model_column_index);
   #endif //GLOM_ENABLE_MAEMO
 
@@ -453,6 +456,10 @@ private:
   bool m_find_mode;
   bool m_allow_only_one_related_record;
 
+  //Used to revert the currently-edited cell.
+  bool m_validation_retry;
+  Glib::ustring m_validation_invalid_text_for_retry;
+
   /// The primary key for the table:
   sharedptr<Field> m_key_field;
 
